#!/usr/bin/env python

"A dictionary example application."

import WebStack.Generic
import XSLForms.Resources
import XSLForms.Utils
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Static import DirectoryResource

# Resource classes.

class DictionaryResource(XSLForms.Resources.XSLFormsResource):

    "A simple resource providing dictionary lookup."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    encoding = "utf-8"
    template_resources = {
        "words" : ("words_template.xhtml", "words_output.xsl")
        }
    in_page_resources = {
        "matches" : ("words_output_entry.xsl", "matches-node"),
        #"word" : ("words_output_word.xsl", "word-node")
        }

    def __init__(self, dict):

        "Initialise the resource with the given 'dict'."

        self.dict = dict

    def respond_to_form(self, trans, form):

        """
        Respond to a request having the given transaction 'trans' and the given
        'form' information.
        """

        in_page_resource = self.get_in_page_resource(trans)
        parameters = form.get_parameters()
        documents = form.get_documents()

        # Ensure the presence of a document.

        if documents.has_key("words"):
            words = documents["words"]
        else:
            words = form.new_instance("words")

        # Add and remove elements according to the selectors found.

        selectors = form.get_selectors()
        XSLForms.Utils.remove_elements(selectors.get("remove"))
        XSLForms.Utils.add_elements(selectors.get("add"), "entry")

        # Ensure all entries have a matches element.
        # Ensure all matches elements have at least one choice.
        # Copy selected matches to their corresponding text field.

        for entry in words.xpath("words/entry"):
            matches_list = entry.xpath("matches")
            if len(matches_list) == 0:
                matches = words.createElement("matches")
                entry.appendChild(matches)
            else:
                matches = matches_list[0]

            if len(entry.xpath("matches/match-enum")) == 0:
                match_enum = words.createElement("match-enum")
                match_enum.setAttribute("word", "")
                matches.appendChild(match_enum)

            if matches.hasAttribute("word"):
                word = matches.getAttribute("word")
                if word != "":
                    entry.setAttribute("word", word)

        # Find requested search locations.

        if selectors.has_key("search"):
            entries = selectors["search"]
        elif in_page_resource == "matches":
            entries = words.xpath("words/entry")
        else:
            entries = []

        # Transform, adding dictionary information.

        for entry in entries:
            word = entry.getAttribute("word")
            if word != "":
                matches = entry.xpath("matches")[0]
                for found_word in self.dict.find(word):
                    match_enum = words.createElement("match-enum")
                    match_enum.setAttribute("word", found_word)
                    matches.appendChild(match_enum)

        # Start the response.

        trans.set_content_type(WebStack.Generic.ContentType("application/xhtml+xml", self.encoding))

        # Define the stylesheet parameters.

        stylesheet_parameters = {}

        # Ensure that an output stylesheet exists.

        if in_page_resource in self.in_page_resources.keys():
            trans_xsl = self.prepare_fragment("words", in_page_resource)
            element_path = parameters.get("element-path", [""])[0]
            stylesheet_parameters["element-path"] = element_path
        else:
            trans_xsl = self.prepare_output("words")

        # Complete the response.

        self.send_output(trans, [trans_xsl], words, stylesheet_parameters)

# Site map initialisation.

def get_site(dict):

    """
    Return a simple Web site resource using the given 'dict' - a dictionary
    employed by the application.
    """

    # Get the main resource and the directory used by the application.

    dictionary_resource = DictionaryResource(dict)
    directory = dictionary_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "scripts" : DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"}),
        # Main page and in-page resources:
        None : dictionary_resource
        })

    return resource

# vim: tabstop=4 expandtab shiftwidth=4
